-- customer_loan_info
DROP TABLE IF EXISTS `customer_loan_info`;
create table `customer_loan_info`
(
    id       bigint auto_increment,
    frame_no varchar(32) not null comment '车架号',
    periods  int(11)     not null comment '贷款期数（月）',
    expires  date        not null comment '贷款到期时间',
    group_id bigint      not null comment '集团id',
    constraint customer_loan_info_pk
        primary key (id)
)
    comment '客户贷款信息';

create unique index customer_loan_info_frame_no
    on customer_loan_info (frame_no);
-- 保有客池
DROP TABLE IF EXISTS `stammkunde_pool`;
create table stammkunde_pool
(
    id              bigint auto_increment,
    customer_id     bigint      not null comment '客户档案id',
    adviser_id      bigint      not null comment '顾问id',
    adviser_name    varchar(64) null comment '顾问名称',
    sources         int(4)      not null comment '渠道来源 10: 新车 20: 外集团 30: 二次分配 40: 公共池 50: 系统分配',
    pool_status     int(3)      not null comment '保有客池状态 1:保有客  2:二次分配 3:公共池',
    aktiv           tinyint(1)  not null comment '活跃数据',
    activation_time datetime    null comment '激活时间 （正式入池时间）',
    reject_time     datetime    null comment '作废时间',
    shop_id         bigint      not null comment '门店id',
    group_id        bigint      not null comment '集团id',
    create_time     datetime    null,
    update_time     datetime    null,
    constraint stammkunde_pool_pk
        primary key (id)
)
    comment '保有客池  记录保有客专属顾问变更历史';

create index stammkunde_pool_adviser_id_index
    on stammkunde_pool (adviser_id);
create index stammkunde_pool_adviser_name_index
    on stammkunde_pool (adviser_name);
create index stammkunde_customer_id_index
    on stammkunde_pool (customer_id);
create index stammkunde_pool_sources_index
    on stammkunde_pool (sources);


-- 客户到店记录
DROP TABLE IF EXISTS `customer_reach_log`;
create table customer_reach_log
(
    id           bigint auto_increment,
    customer_id  bigint       not null comment '档案id',
    frame_no     varchar(32)  not null comment '车架号',
    shop_id      bigint       not null comment '门店id',
    advisor_id   bigint       null comment '顾问id',
    advisor_name varchar(255) null comment '顾问名称',
    arrival_time datetime     not null comment '到店时间',
    group_id     bigint       not null comment '集团id',
    constraint customer_reach_log
        primary key (id)
)
    comment '客户到店记录';

create index customer_id_frame_no_index
    on customer_reach_log (customer_id, frame_no);

-- 客户线索池
DROP TABLE IF EXISTS `customer_clue_pool`;
-- auto-generated definition
create table customer_clue_pool
(
    id                 bigint auto_increment
        primary key,
    referer_id         bigint      not null comment '档案id或者事故车线索id',
    frame_no           varchar(32) null comment '车架号 (非事故车类型not null)',
    plate_no           varchar(32) not null comment '车牌号',
    clue_type          tinyint(3)  not null comment '线索类型 同跟进类型',
    add_time           datetime    not null comment '线索添加时间',
    start_time         datetime    not null comment '生效时间',
    deadline           datetime    not null comment '线索过期时间',
    close_time         datetime    null comment '关闭时间',
    clue_status        int(3)      not null comment '状态 1:等待  2: 进行中  3: 已完成 4：已战败',
    redistribution     tinyint(1)  not null comment '是否已经二次分配',
    original_user_id   bigint      not null comment '初始跟进人员id',
    original_user_name varchar(20) null comment '初始跟进人名称',
    original_shop_id   bigint      not null comment '初始门店ID',
    original_shop_name varchar(64) null comment '初始门店名称',
    finish_user_id     bigint      null comment '成交人员id',
    finish_user_name   varchar(32) null comment '成交人员名称',
    finish_shop_id     bigint      null comment '成交门店id',
    finish_shop_name   varchar(64) null comment '成交门店名称',
    group_id           bigint      not null comment '集团id',
    create_time        datetime    null,
    update_time        datetime    null
)
    comment '客户线索池';

create index customer_id_frame_no_plate_no_index
    on customer_clue_pool (referer_id, frame_no, plate_no);



DROP TABLE IF EXISTS `customer_retention_ratio`;
create table customer_retention_ratio
(
    id          bigint auto_increment,
    user_id     bigint       not null comment '顾问id',
    user_name   varchar(255) not null comment '顾问名称',
    dimensions  tinyint(1)   not null comment '模式 1 人员 2服务站',
    shop_id     bigint       not null comment '服务站id',
    shop_name   varchar(255) not null comment '服务站名称',
    initial_num int          not null comment '期初保有客数',
    defeat_num  int          not null comment '近6个月战败数',
    data_date   date         not null comment '数据日期',
    constraint customer_retention_ratio_pk
        primary key (id)
)
    comment '保有客保持率';


DROP TABLE IF EXISTS `customer_import`;
create table customer_import
(
    id              bigint auto_increment
        primary key,
    plate_no        varchar(32)          null,
    adviser_name    varchar(32)          null,
    adviser_id      bigint               null,
    brand_name      varchar(32)          null,
    series_name     varchar(64)          null,
    spec_name       varchar(128)         null,
    spec_code       varchar(32)          null,
    name            varchar(32)          null,
    shop_id         bigint               null,
    engine_no       varchar(32)          null,
    mobile          varchar(32)          not null,
    current_mileage int        default 0 not null,
    arrival_time    datetime             not null,
    frame_no        varchar(32)          not null,
    mark            int(3) default 0 not null,
    need_assigning  tinyint(1) default 0 not null,
    old_id          bigint               null
)
    comment '档案导入表';

create index customer_import_frame_no_index
    on customer_import (frame_no);

create index customer_import_oldId_index
    on customer_import (old_id);


-- auto-generated definition
DROP TABLE IF EXISTS `follow_task`;
create table follow_task
(
    id               bigint auto_increment
        primary key,
    clue_id          bigint               not null comment '线索id',
    customer_id      bigint               not null comment '客户id 事故车的情况下是事故池的id',
    type             int(3)               not null comment '设置类型 1：首保  2：例保 3：事故车 4：续保 等',
    begin_time       datetime             null comment '任务开始时间',
    redistribution   tinyint(1) default 0 not null comment '二次分配',
    deadline         datetime             null,
    state            int(3)               not null comment '完成状态 1：进行中 2：已完成 3：已战败',
    close_time       datetime             null comment '任务终止时间',
    follow_user      bigint               null comment '当前跟进人',
    follow_user_name varchar(64)               null comment '当前跟进人名称',
    follow_shop      bigint               null comment '当前跟进服务站',
    finish_user      bigint               null comment '完成人',
    finish_user_name varchar(64)               null comment '完成人名称',
    finish_shop      bigint               null comment '完成门店',
    reason           int(3)               null comment '战败原因',
    group_id         bigint               not null,
    create_time      datetime             not null,
    update_time      datetime             null
)
    comment '跟进任务记录';

create index customer_id_index
    on follow_task (customer_id);



alter table follow_record
    add limit_time datetime null comment '待办截止时间（事故车才有）' after remark;

alter table follow_record
    drop column dealer_id;

alter table follow_record
    add user_name varchar(255) null comment '跟进人员姓名' after user_id;



alter table follow_notice_record
    change task_id clue_id bigint not null comment '线索id';



rename table follow_record_attachments to follow_record_log;


alter table original_data
    add user_id bigint null comment '业务产生人员id' after plate_no;
alter table original_data
    add shop_id bigint null comment '门店id' after solved;

alter table customer
    add arrival_time datetime null comment '上次进站时间' after vehicle_license;

alter table customer
    modify current_mileage int default 0 null comment '上次进站里程 单位：KM';

alter table customer
    add insurance_expires datetime null comment '保险过期时间' after current_mileage;


/**
  贷款信息处理
 */

-- 新增贷款信息
insert into customer_loan_info (frame_no, periods, expires, group_id)
SELECT t1.vin frame_no,
       t1.periods,
       t1.expires,
       2
FROM (SELECT * FROM loan_info t1 GROUP BY t1.vin) t1;


-- auto-generated definition
DROP TABLE IF EXISTS `cas_015_d`;
create table cas_015_d
(
    id               bigint auto_increment comment '主键'
        primary key,
    report_date      date                    not null comment '日期',
    report_dimension varchar(2)              not null comment '维度',
    group_id         bigint                  not null comment '集团id',
    group_name       varchar(32)             not null comment '集团名称',
    shop_id          bigint      default -1  not null comment '门店id',
    shop_name        varchar(64) default '-' not null comment '门店名称',
    data_time        datetime                not null comment '数据时间',
    t_id             varchar(32)             not null comment '目标id',
    t_name           varchar(32)             not null comment '目标名称',
    ex_id            varchar(32) default '-' not null comment '扩展id',
    ex_name          varchar(32) default '-' not null comment '扩展名称',
    type             int                     not null comment '类型 1：首保 2：例保 3：事故 4：续保',
    i1               int                     not null comment '跟进待办任务数',
    i2               int                     not null comment '待跟进数',
    i3               int                     not null comment '及时跟进数',
    i4               int                     not null comment '逾期数',
    i5               int                     null comment '跟进任务数',
    i6               int                     null comment '跟进任务成交数',
    i7               int                     null comment '二次分配数',
    i8               int                     null comment '二次分配成交数',
    d1               decimal(15, 2)          null comment '成交任务完成率'
)
    comment '售后-CRM-日报';


-- auto-generated definition
DROP TABLE IF EXISTS `cas_015_m`;
create table cas_015_m
(
    id               bigint auto_increment comment '主键'
        primary key,
    report_date      date                    not null comment '日期',
    report_dimension varchar(2)              not null comment '维度',
    group_id         bigint                  not null comment '集团id',
    group_name       varchar(32)             not null comment '集团名称',
    shop_id          bigint      default -1  not null comment '门店id',
    shop_name        varchar(64) default '-' not null comment '门店名称',
    data_time        datetime                not null comment '数据时间',
    t_id             varchar(32)             not null comment '目标id',
    t_name           varchar(32)             not null comment '目标名称',
    ex_id            varchar(32) default '-' not null comment '扩展id',
    ex_name          varchar(32) default '-' not null comment '扩展名称',
    type             int                     not null comment '类型 1：首保 2：例保 3：事故 4：续保',
    i1               int                     not null comment '跟进待办任务数',
    i2               int                     not null comment '待跟进数',
    i3               int                     not null comment '及时跟进数',
    i4               int                     not null comment '逾期数',
    i5               int                     null comment '跟进任务数',
    i6               int                     null comment '跟进任务成交数',
    i7               int                     null comment '二次分配数',
    i8               int                     null comment '二次分配成交数',
    d1               decimal(15, 2)          null comment '成交任务完成率'
)
    comment '售后-CRM-月报';

