package cn.fw.valhalla.domain.dto;

import cn.fw.valhalla.common.enums.AllocationTypeEnum;
import lombok.Data;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author : kurisu
 * @className : LeaveAllocationDTO
 * @description : 离职分配DTO
 * @date: 2020-10-16 17:52
 */
@Data
public class LeaveAllocationDTO {
    @NotNull(message = "记录id不能为空")
    private Long id;
    /**
     * 分配规则
     */
    @Valid
    @NotEmpty(message = "分配规则不能为空")
    private List<Allocation> ruleList;


    /**
     * 原始顾问id （前端无关）
     */
    private Long adviserId;

    public static class Allocation {
        /**
         * 指定人员id
         */
        @NotNull(message = "人员id不能为空")
        private Long userId;
        /**
         * 人员名称
         */
        private String userName;
        /**
         * 指定人员id
         */
        @Min(value = 1, message = "保有客数量必须大于0")
        private Integer num;

        /**
         * 门店id （前端无关）
         */
        private Long shopId;

        public Long getUserId() {
            return userId;
        }

        public void setUserId(Long userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public Integer getNum() {
            return num;
        }

        public void setNum(Integer num) {
            this.num = num;
        }

        public Long getShopId() {
            return shopId;
        }

        public void setShopId(Long shopId) {
            this.shopId = shopId;
        }
    }
}
