package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ApproveStateEnum
 * @description : 审批状态
 * @date: 2020-08-11 17:37
 */
public enum ApproveStateEnum implements IEnum<Integer> {
    /**
     * 等待回调
     */
    WAIT(1, "等待回调"),
    /**
     * 完成回调
     */
    DONE(2, "完成回调"),
    /**
     * 已取消
     */
    CANCELED(3, "已取消"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ApproveStateEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ApproveStateEnum ofValue(final Integer value) {
        for (final ApproveStateEnum typeEnum : ApproveStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ApproveStateEnum typeEnum : ApproveStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
