package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : AttTypeEnum
 * @description : 附件类型
 * @date: 2020-08-11 17:37
 */
public enum AttTypeEnum implements IEnum<Integer> {
    /**
     * 电话
     */
    PHONE(1, "电话"),
    /**
     * 微聊
     */
    IMCHAT(2, "微聊"),
    /**
     * 拜访
     */
    VISIT(3, "拜访"),
    /**
     * 微信
     */
    WECHAT(4, "微信"),
    /**
     * 短信
     */
    SMS(5, "短信"),
    /**
     * 企业微信
     */
    EWECHAT(6, "企业微信"),
    /**
     * 智能通话
     */
    SMART_PHONE(7, "智能通话"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    AttTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static AttTypeEnum ofValue(final Integer value) {
        for (final AttTypeEnum typeEnum : AttTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final AttTypeEnum typeEnum : AttTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
