package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ContactRelationEnum
 * @description : 联系人关系
 * @date: 2020-08-11 17:37
 */
public enum ContactRelationEnum implements IEnum<Integer> {
    /**
     * 父母
     */
    PARENTS(1, "父母"),
    /**
     * 子女
     */
    CHILDREN(2, "子女"),
    /**
     * 夫妻
     */
    MARRIAGE(3, "夫妻"),
    /**
     * 亲戚
     */
    RELATIVES(4, "亲戚"),
    /**
     * 朋友
     */
    FRIENDS(5, "朋友"),
    /**
     * 同事
     */
    COLLEAGUE(6, "同事"),
    /**
     * 本人
     */
    SELF(7, "本人"),
    /**
     * 其他
     */
    OTHER(8, "其他");

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ContactRelationEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ContactRelationEnum ofValue(final Integer value) {
        for (final ContactRelationEnum typeEnum : ContactRelationEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ContactRelationEnum typeEnum : ContactRelationEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
