package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ContactTypeEnum
 * @description : 联系人类型
 * @date: 2020-08-11 17:37
 */
public enum ContactTypeEnum implements IEnum<Integer> {
    /**
     * 送修人
     */
    SENDER(1, "送修人"),
    /**
     * 同行人
     */
    OTHER(2, "同行人"),
    /**
     * 车主
     */
    OWNER(3, "车主");

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ContactTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ContactTypeEnum ofValue(final Integer value) {
        for (final ContactTypeEnum typeEnum : ContactTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ContactTypeEnum typeEnum : ContactTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
