package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : CustomerChangeTypeEnum
 * @description : 战败类型
 * @date: 2020-08-11 17:37
 */
public enum CustomerChangeTypeEnum implements IEnum<Integer> {
    /**
     * 战败
     */
    DEFEAT(1, "战败"),
    /**
     * 划走
     */
    CHANGE(2, "划走"),
    /**
     * 其他
     */
    OTHER(3, "其他"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    CustomerChangeTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static CustomerChangeTypeEnum ofValue(final Integer value) {
        for (final CustomerChangeTypeEnum typeEnum : CustomerChangeTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final CustomerChangeTypeEnum typeEnum : CustomerChangeTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
