package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : AttTypeEnum
 * @description : 附件类型
 * @date: 2020-08-11 17:37
 */
public enum FeedbackTypeEnum implements IEnum<Integer> {
    /**
     * 距离远
     */
    FAR(1, "距离远"),
    /**
     * 微聊
     */
    EXPENSIVE(2, "收费高"),
    /**
     * 过几天进站
     */
    WAITING(3, "过几天进站"),
    /**
     * 微信
     */
    OTHER(99, "其他"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    FeedbackTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static FeedbackTypeEnum ofValue(final Integer value) {
        for (final FeedbackTypeEnum typeEnum : FeedbackTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final FeedbackTypeEnum typeEnum : FeedbackTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
