package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : PublicClueStateEnum
 * @description : 公共池客户转换状态
 * @date: 2021-03-29 17:37
 */
public enum PublicClueStateEnum implements IEnum<Integer> {
    /**
     * 进行中
     */
    ONGOING(1, "进行中"),
    /**
     * 已完成
     */
    COMPLETE(2, "已完成"),
    /**
     * 已失败
     */
    DEFEAT(3, "已失败");

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    PublicClueStateEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static PublicClueStateEnum ofValue(final Integer value) {
        for (final PublicClueStateEnum typeEnum : PublicClueStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @Override
    @JsonValue
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final PublicClueStateEnum typeEnum : PublicClueStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
