package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : SettingUnitEnum
 * @description : 设置单位
 * @date: 2020-08-11 17:37
 */
public enum SettingUnitEnum implements IEnum<Integer> {
    /**
     * 次
     */
    TIMES(1, "次"),
    /**
     * 分钟
     */
    MINUTE(2, "分钟"),
    /**
     * 小时
     */
    HOUR(3, "小时"),
    /**
     * 天
     */
    DAY(4, "天"),
    /**
     * 月
     */
    MONTH(5, "月"),
    /**
     * 百分比
     */
    RATIO(9, "%"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    SettingUnitEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SettingUnitEnum ofValue(final Integer value) {
        for (final SettingUnitEnum typeEnum : SettingUnitEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SettingUnitEnum typeEnum : SettingUnitEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
