package cn.fw.valhalla.domain.query;

import cn.fw.common.page.BasePageQuery;
import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.valhalla.common.utils.DateUtil;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 档案查询条件
 *
 * @author kurisu
 */

@Data
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
public class CustomerQueryVO extends BasePageQuery {
    /**
     * 允许通过车主姓名、车架号、车牌号匹配
     */
    private String keyword;
    /**
     * 工作单位
     */
    private String companyName;
    /**
     * 区域code
     */
    private String regionCode;
    /**
     * 客户登记
     */
    private List<String> levels = Collections.emptyList();
    /**
     * 标签
     */
    private List<String> tags = Collections.emptyList();
    /**
     * 建档时间(开始)
     */
    private Long leftCreatTime;
    /**
     * 建档时间(结束)
     */
    private Long rightCreatTime;
    /**
     * 品牌id
     */
    private Long brandId;
    /**
     * 车系id
     */
    private Long seriesId;
    /**
     * 车系id
     */
    private Long specId;
    /**
     * 车主出生月份
     */
    private Integer month;
    /**
     * 车主性别
     */
    private Integer gender;
    /**
     * 车主最小年龄
     */
    private Integer minAge;
    /**
     * 车主最大年龄
     */
    private Integer maxAge;
    /**
     * 客户类型;1:个人；2:单位
     */
    private Integer cusType;

    /**
     * 时间范围
     */
    private String timeHorizon;
    /**
     * 是否查询进行中的任务
     */
    private Boolean underway;
    /**
     * 跟进类型
     */
    private Integer followType;

    /**
     * 中心点纬度
     */
    private BigDecimal lat;

    /**
     * 中心点经度
     */
    private BigDecimal lng;

    /**
     * 半径距离(米)
     */
    private Double radius;
    /**
     * 来自报表
     */
    private Boolean fromReport;
    /**
     * 所属服务站id￿
     */
    private String shopIds;
    /**
     * 集团id
     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;
    /**
     * 专属顾问id
     */
    private Long consultantId;

    //内部处理
    private List<Long> customerIdList;

    public Date getLeftCreatTime() {
        if (this.leftCreatTime != null) {
            return new Date(this.leftCreatTime);
        }
        return null;
    }

    public Date getRightCreatTime() {
        if (this.rightCreatTime != null) {
            return new Date(this.rightCreatTime);
        }
        return null;
    }

    public Date getStartBirthday() {
        if (minAge != null) {
            return DateUtil.getNowExpiredYear(minAge * -1);
        }
        return null;
    }

    public Date getEndBirthday() {
        if (maxAge != null) {
            return DateUtil.getNowExpiredYear(maxAge * -1);
        }
        return null;
    }

    public List<Long> getShopIds() {
        if (StringUtils.isBlank(shopIds)) {
            return null;
        }
        List<Long> list = Arrays.stream(shopIds.split(",")).filter(StringUtils::isNoneBlank).map(Long::valueOf).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list;
    }

    public String getShopIdStr() {
        return shopIds;
    }
}