package cn.fw.valhalla.domain.query;

import cn.fw.common.page.BasePageQuery;
import cn.fw.common.web.annotation.LoginContextField;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : PoolQuery
 * @description : 基础查询条件
 * @date: 2020-11-20 11:42
 */
@Data
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
public class PoolQuery extends BasePageQuery {
    protected final static int TIME_STR_LENGTH = 2;
    private String shopIds;
    private Long userId;
    private String userName;
    private String plateNo;
    /**
     * 车架号
     */
    private String frameNo;
    /**
     * 1升序 2降序
     */
    private Integer order;
    private String orderAtt;


    /**
     * 服务端自己处理的参数
     */
    private String orderString;
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    public List<Long> getShopIds() {
        if (StringUtils.isBlank(shopIds)) {
            return null;
        }
        List<Long> list = Arrays.stream(shopIds.split(",")).filter(StringUtils::isNoneBlank).map(Long::valueOf).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list;
    }
}
