package cn.fw.valhalla.domain.query;

import cn.fw.valhalla.common.utils.DateUtil;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : PublicCluePoolQueryVO
 * @description : 公共客户线索池查询条件
 * @date: 2021-03-29 14:55
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class PublicCluePoolQueryVO extends PoolQuery {
    private Long brandId;
    private Long seriesId;
    private Long specId;
    /**
     * 档案创建时间
     */
    private String activationTime;
    private String customerName;
    private String address;
    private String regionCode;
    private Integer level;
    private Boolean warrantyCard;
    private Boolean maintainCard;
    /**
     * 贷款期客户
     */
    private Boolean loanCustomer;


    public Date getStartTime1() {
        if (StringUtils.isBlank(activationTime)) {
            return null;
        }
        String[] times = activationTime.split(",");
        if (StringUtils.isNotBlank(times[0]) && NumberUtils.isDigits(times[0])) {
            LocalDateTime localDateTime = Instant.ofEpochMilli(NumberUtils.toLong(times[0])).atZone(ZoneId.systemDefault()).toLocalDateTime();
            return DateUtil.getBeginInTime(DateUtil.localDateTime2Date(localDateTime));
        }
        return null;
    }

    public Date getStartTime2() {
        if (StringUtils.isBlank(activationTime)) {
            return null;
        }
        String[] times = activationTime.split(",");
        if (times.length < TIME_STR_LENGTH) {
            return null;
        }
        if (StringUtils.isNotBlank(times[1]) && NumberUtils.isDigits(times[1])) {
            LocalDateTime localDateTime = Instant.ofEpochMilli(NumberUtils.toLong(times[1])).atZone(ZoneId.systemDefault()).toLocalDateTime();
            return DateUtil.getEndInTime(DateUtil.localDateTime2Date(localDateTime));
        }
        return null;
    }


    public Integer getWarrantyCard() {
        if (Objects.isNull(warrantyCard)) {
            return null;
        }
        return warrantyCard ? 1 : 0;
    }

    public Integer getMaintainCard() {
        if (Objects.isNull(maintainCard)) {
            return null;
        }
        return maintainCard ? 1 : 0;
    }

    public Integer getLoanCustomer() {
        if (Objects.isNull(loanCustomer)) {
            return null;
        }
        return loanCustomer ? 1 : 0;
    }
}
