package cn.fw.valhalla.domain.query;

import cn.fw.common.web.annotation.LoginContextField;
import lombok.Data;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : StammkundeAnalyseQueryVO
 * @description : 保有客分析查询条件
 * @date: 2021-01-21 15:40
 */
@Data
@ToString
public class StammkundeAnalyseQueryVO {
    /**
     * 时间范围
     */
    private String timeHorizon;
    /**
     * 是否查询进行中的任务
     */
    private Boolean underway;
    /**
     * 跟进类型
     */
    private Integer followType;

    /**
     * 中心点纬度
     */
    private BigDecimal lat;

    /**
     * 中心点经度
     */
    private BigDecimal lng;

    /**
     * 半径距离(米)
     */
    private Double radius;

    /**
     * 门店范围
     */
    private String shopIds;

    //下面的是业务处理是需要的参数

    /**
     * 纬度范围
     */
    private BigDecimal minLat;
    private BigDecimal maxLat;

    /**
     * 经度范围
     */
    private BigDecimal minLng;
    private BigDecimal maxLng;

    private Date startTime;
    private Date endTime;

    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;
    private Set<Long> customerIds;

    public boolean hasRange() {
        return Objects.nonNull(lat) && Objects.nonNull(lng) && Objects.nonNull(radius) && radius > 0D;
    }

    public List<Long> getShopIds() {
        if (StringUtils.isBlank(shopIds)) {
            return null;
        }
        List<Long> list = Arrays.stream(shopIds.split(",")).filter(StringUtils::isNoneBlank).map(Long::valueOf).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list;
    }
}
