package cn.fw.valhalla.domain.vo.follow;

import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import lombok.Data;

import java.util.Date;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : CustomerClueHistoryVO
 * @description : 客户跟进线索历史记录
 * @date: 2021-03-29 16:48
 */
@Data
public class CustomerClueHistoryVO {
    /**
     * id
     */
    private Long id;
    /**
     * 跟进类型
     */
    private Integer type;
    /**
     * 原始门店信息
     */
    private String shopName;
    /**
     * 原始跟进人
     */
    private String userName;
    /**
     * 到期时间
     */
    private Date deadline;

    public boolean getOverdue() {
        return deadline.before(new Date());
    }

    public static CustomerClueHistoryVO with(CustomerCluePool db) {
        if (Objects.isNull(db)) {
            return null;
        }
        CustomerClueHistoryVO vo = new CustomerClueHistoryVO();
        vo.setId(db.getId());
        vo.setType(db.getClueType().getValue());
        vo.setShopName(db.getOriginalShopName());
        vo.setUserName(db.getOriginalUserName());
        vo.setDeadline(db.getDeadline());
        return vo;
    }

    public static CustomerClueHistoryVO with(PublicCluePool db) {
        if (Objects.isNull(db)) {
            return null;
        }
        CustomerClueHistoryVO vo = new CustomerClueHistoryVO();
        vo.setId(db.getId());
        vo.setType(FollowTypeEnum.OT.getValue());
        vo.setShopName(db.getShopName());
        vo.setUserName(db.getAdviserName());
        vo.setDeadline(db.getDeadline());
        return vo;
    }
}
