package cn.fw.valhalla.rpc.cas;

import cn.fw.cas.sdk.api.CasApiService;
import cn.fw.cas.sdk.result.DutySubstituteDto;
import cn.fw.data.base.domain.common.Message;
import cn.fw.valhalla.rpc.cas.dto.SubstituteDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author : kurisu
 * @className : CasRpcService
 * @description : 售后rpc接口
 * @date: 2020-08-12 17:34
 */
@Slf4j
@Service
public class CasRpcService {
    private final CasApiService casApiService;

    public CasRpcService(final CasApiService casApiService) {
        this.casApiService = casApiService;
    }

    /**
     * 查询服务顾问代班人
     *
     * @param shopId
     * @param userId
     * @return
     */
    public SubstituteDTO getSubstitute(Long shopId, Long userId) {
        try {
            Message<DutySubstituteDto> msg = casApiService.getAdvisorSubstitute(shopId, userId);
            log.info("调用售后查询服务顾问代班人接口状态: {}", msg.isSuccess());
            if (!msg.isSuccess()) {
                return null;
            }
            DutySubstituteDto data = msg.getData();
            log.info("查询服务顾问代班人 data[{}]", data);
            if (Objects.isNull(data)) {
                return null;
            }
            SubstituteDTO dto = new SubstituteDTO();
            BeanUtils.copyProperties(data, dto);
            return dto;
        } catch (Exception e) {
            log.error("查询服务顾问代班人失败", e);
            return null;
        }
    }

    public int getPullinTimes(final Long groupId, final Long customerId) {
        try {
            Message<Integer> msg = casApiService.queryPullinTimes(groupId, customerId);
            log.info("调用售后查询进站次数接口状态: {}", msg.isSuccess());
            if (!msg.isSuccess()) {
                return 0;
            }
            Integer data = msg.getData();
            log.info("查询进站次数 data[{}]", data);
            if (Objects.isNull(data)) {
                return 0;
            }
            return data;
        } catch (Exception e) {
            log.error("查询进站次数失败", e);
            return 0;
        }
    }
}
