package cn.fw.valhalla.rpc.dfs;

import cn.fw.valhalla.rpc.dfs.dto.DfsFileDTO;
import cn.fw.data.base.domain.common.Message;
import cn.fw.dfs.sdk.api.FileService;
import cn.fw.dfs.sdk.api.enums.InvokerSystemEnum;
import cn.fw.dfs.sdk.api.result.DfsFile;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * fw-cas
 * <p>
 * create at 2019-04-02
 *
 * @author kurisu
 */
@Slf4j
@Service
public class DfsService {

    @Autowired
    private FileService fileService;


    public String upload(byte[] file, String fileName, String contentType) {
        try {
            Message msg = fileService.upload(file, fileName, contentType, InvokerSystemEnum.AFTER_SALE.getSysName());
            if (!msg.isSuccess()) {
                log.error("调用DFS系统上传文件失败：file{},fileName{},contentType{},result{}",
                    file, fileName, contentType, msg.getResult());
                return null;
            }
            return (String) msg.getData();
        } catch (Exception e) {
            log.error("调用DFS系统上传文件失败：file{},fileName{},contentType{},e{}", file, fileName, contentType, e);
            return null;
        }

    }

    public DfsFileDTO download(String fid) {
        try {
            Message<DfsFile> msg = fileService.download(fid);
            if (!msg.isSuccess()) {
                return null;
            }
            DfsFileDTO dto = new DfsFileDTO();
            BeanUtils.copyProperties(msg.getData(), dto);
            return dto;
        } catch (Exception e) {
            log.error("调用DFS失败：", e);
            return null;
        }
    }
}
