package cn.fw.valhalla.rpc.hestia;

import cn.fw.data.base.domain.common.Message;
import cn.fw.hestia.sdk.api.IMessageCenterService;
import cn.fw.hestia.sdk.params.TemplateMessageParam;
import cn.fw.hestia.sdk.result.SendResult;
import cn.fw.valhalla.rpc.hestia.dto.TemplateDTO;
import cn.fw.valhalla.rpc.hestia.dto.TemplateResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author : kurisu
 * @className : HestiaRpcService
 * @description : c端消息中心rpc
 * @date: 2021-09-30 15:23
 */
@Slf4j
@Service
public class HestiaRpcService {
    private final IMessageCenterService messageCenterService;

    @Autowired
    public HestiaRpcService(final IMessageCenterService messageCenterService) {
        this.messageCenterService = messageCenterService;
    }

    public TemplateResult senTemplateMsg(TemplateDTO templateDTO) {
        log.info("开始发送模板通知，senderId：{}", templateDTO);
        try {

            TemplateMessageParam messageParam = new TemplateMessageParam();
            BeanUtils.copyProperties(templateDTO, messageParam);
            Message<SendResult> result = messageCenterService.send(messageParam);
            SendResult resultData = result.getData();
            if (!result.isSuccess() || Objects.isNull(resultData)) {
                log.error("【hestia系统】发送模板消息失败：{}", result.getResult());
                return null;
            }
            TemplateResult res = new TemplateResult();
            BeanUtils.copyProperties(resultData, res);
            return res;
        } catch (Exception e) {
            log.error("发送模板消息失败", e);
            return null;
        }
    }
}
