package cn.fw.valhalla.rpc.order;

import cn.fw.data.base.domain.common.Message;
import cn.fw.order.sdk.api.CarOrderApiService;
import cn.fw.order.sdk.result.CarOrderResultDto;
import cn.fw.valhalla.rpc.AbsBaseRpcService;
import cn.fw.valhalla.rpc.order.dto.NewCarOrderInfo;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * OrderRpcService
 * create at 2020-12-17
 *
 * @author kurisu
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class OrderRpcService extends AbsBaseRpcService {

    private final CarOrderApiService carOrderApiService;

    @Value("${spring.cache.custom.global-prefix}:rpc:newcar:order")
    private String keyPrefix;

    /**
     * 查询vin对应的订单信息
     *
     * @param vin 车架号
     */
    public NewCarOrderInfo getOrderInfo(final String vin) {
        final String message = "查询vin对应的订单信息[getOrderInfo]";
        try {
            final String key = generateKey(vin);
            NewCarOrderInfo orderInfo = getFromCache(key, NewCarOrderInfo.class);
            if (Objects.nonNull(orderInfo)) {
                return orderInfo;
            }
            log.info("{}，vin:{}", message, vin);
            Message<CarOrderResultDto> msg = carOrderApiService.getCarOrderByVin(vin);
            log.info("{},vin:{}，result:{}", message, vin, JSON.toJSONString(msg));
            CarOrderResultDto data = msg.getData();
            if (!msg.isSuccess() || Objects.isNull(data)) {
                return null;
            }
            NewCarOrderInfo order = new NewCarOrderInfo();
            BeanUtils.copyProperties(data, order);
            setToCache(key, JSONObject.toJSONString(order), 60 * 60 * 24);
            return order;
        } catch (Exception ex) {
            log.error("{}失败,vin:{}", message, vin, ex);
        }
        return null;
    }

    @Override
    protected String getKeyPrefix() {
        return keyPrefix;
    }

    private String generateKey(final String vin) {
        BV.isNotBlank(vin, "vin cannot be blank");

        return String.format("%s:%s", getKeyPrefix(), vin);
    }
}
