package cn.fw.valhalla.controller.api;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.valhalla.domain.dto.OriginalDataDTO;
import cn.fw.valhalla.sdk.api.FollowApiService;
import cn.fw.valhalla.sdk.enums.DataTypeEnum;
import cn.fw.valhalla.sdk.param.FollowOriginalParam;
import cn.fw.valhalla.service.bus.follow.OriginalDataBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @className : FollowApiServiceImpl
 * @description : 跟进api
 * @date: 2020-08-14 17:53
 */
@Slf4j
@RestController
@RequestMapping("/api/valhalla/follow")
public class FollowApiServiceImpl implements FollowApiService {

    private final OriginalDataBizService originalDataBizService;

    @Autowired
    public FollowApiServiceImpl(final OriginalDataBizService originalDataBizService) {
        this.originalDataBizService = originalDataBizService;
    }

    @Override
    @PostMapping("/origin/save")
    @ControllerMethod("业务元数据保存")
    public Message<Void> saveOriginal(@Valid @RequestBody FollowOriginalParam originalParam) {
        OriginalDataDTO dataDTO = new OriginalDataDTO();
        BeanUtils.copyProperties(originalParam, dataDTO);
        originalDataBizService.saveOrigin(dataDTO);
        return success();
    }

    @Override
    @GetMapping("/origin/remove")
    @ControllerMethod("业务元数据删除")
    public Message<Void> removeData(@RequestParam("type") Integer type, @RequestParam("detailId") String detailId) {
        DataTypeEnum typeEnum = DataTypeEnum.ofValue(type);
        BV.notNull(typeEnum, "业务类型不正确");
        originalDataBizService.remove(typeEnum, detailId);
        return success();
    }
}
