package cn.fw.valhalla.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.SettingDTO;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.service.bus.setting.SettingBizService;
import com.alibaba.fastjson.JSONArray;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.valhalla.common.constant.MessageStr.QUERY_FAILURE;
import static cn.fw.valhalla.common.constant.MessageStr.SAVE_FAILURE;

/**
 * @author : kurisu
 * @className : FollowSettingController
 * @description : 跟进设置控制器
 * @date: 2020-08-11 16:24
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/setting")
public class FollowSettingController {

    private final SettingBizService settingBizService;

    @Autowired
    public FollowSettingController(final SettingBizService settingBizService) {
        this.settingBizService = settingBizService;
    }

    /**
     * 查询配置信息
     *
     * @param currentUser
     * @param category
     * @return
     */
    @GetMapping("/list")
    @ControllerMethod("查询配置列表")
    public Message<List<SettingVO>> getList(@CurrentUser LoginAuthBean currentUser,
                                            @NotNull(message = "跟进类型不能为空") final Integer category) {
        List<SettingVO> list = settingBizService.querySettingByCat(category, currentUser.getGroupId());
        return success(list);
    }

    /**
     * 保存设置
     *
     * @param currentUser
     * @param list
     * @return
     */
    @PostMapping("/save")
    @ControllerMethod("保存设置")
    public Message<Boolean> save(@CurrentUser LoginAuthBean currentUser,
                                 @RequestBody @NotEmpty(message = "设置项不能为空") @Valid final List<SettingDTO> list) {
        return success(settingBizService.saveSetting(list, currentUser.getGroupId()));
    }
}
