package cn.fw.valhalla.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.DistributableDTO;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.enums.LeaveTodoTypeEnum;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.DistributableVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.service.bus.LeaveNeedDoBizService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.valhalla.common.constant.MessageStr.QUERY_FAILURE;
import static cn.fw.valhalla.common.constant.MessageStr.SAVE_FAILURE;

/**
 * @author : kurisu
 * @className : LeaveNeedDoController
 * @description : 人员变动待处理事件
 * @date: 2020-08-15 14:52
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/leave2do")
public class LeaveNeedDoController {

    private final LeaveNeedDoBizService leaveNeedDoBizService;

    @Autowired
    public LeaveNeedDoController(final LeaveNeedDoBizService leaveNeedDoBizService) {
        this.leaveNeedDoBizService = leaveNeedDoBizService;
    }

    @GetMapping("/customer/list")
    @ControllerMethod("保有客档案待分配人员列表")
    public Message<AppPage<LeaveNeedDoVO>> save(@CurrentUser LoginAuthBean currentUser, final LeaveQueryVO queryVO) {
        queryVO.setType(LeaveTodoTypeEnum.CUSTOMER.getValue());
        return success(leaveNeedDoBizService.getList(currentUser, queryVO));
    }


    @PostMapping("/customer/allocation")
    @ControllerMethod("保有客档案分配")
    public Message<Void> allocation(@CurrentUser LoginAuthBean currentUser, @RequestBody @Valid LeaveAllocationDTO dto) {
        leaveNeedDoBizService.allocation(currentUser, dto);
        return success();
    }

    @GetMapping("/customer/distributable")
    @ControllerMethod("查询门店可分配顾问列表")
    public Message<List<DistributableVO>> distributableList(@NotNull(message = "门店id不能为空") final Long shopId) {
        return success(leaveNeedDoBizService.distributableList(shopId));
    }

    @PostMapping("/customer/distributable/do")
    @ControllerMethod("保有客主动分配")
    public Message<Void> distributableDo(@CurrentUser LoginAuthBean currentUser, @RequestBody @Valid DistributableDTO dto) {
        leaveNeedDoBizService.activeAllocation(currentUser, dto);
        return success();
    }
}
