package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.CustomerCluePoolService;
import cn.fw.valhalla.service.data.FollowTaskService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowTask
 * @description : 跟进任务处理
 * @date: 2020-08-24 16:32
 */
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class FollowTaskDealTask {
    private final FollowTaskService followTaskService;
    private final FollowBizService followBizService;
    private final CustomerCluePoolService customerCluePoolService;

    @Autowired
    public FollowTaskDealTask(final FollowTaskService followTaskService,
                              final FollowBizService followBizService,
                              final CustomerCluePoolService customerCluePoolService) {
        this.followTaskService = followTaskService;
        this.followBizService = followBizService;
        this.customerCluePoolService = customerCluePoolService;
    }

    /**
     * 开始任务
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 10)
    @Transactional(rollbackFor = Exception.class)
    public void startClue() {
        List<CustomerCluePool> list = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getClueStatus, ClueStatusEnum.WAITING)
                .ge(CustomerCluePool::getStartTime, DateUtil.startDate(new Date()))
                .le(CustomerCluePool::getStartTime, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .last("limit 0, 500")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (CustomerCluePool cluePool : list) {
            followBizService.startClue(cluePool);
        }
    }

    /**
     * 结束任务
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void endTask() {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .le(FollowTask::getDeadline, DateUtil.localDateTime2Date(LocalDateTime.now()))
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowTask r : list) {
            followBizService.endTask(r);
        }
    }
}
