package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.OriginalDataService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @author : kurisu
 * @className : OriginDataDealTask
 * @description : 原始数据处理
 * @date: 2020-08-18 09:29
 */
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class OriginDataDealTask {
    private final OriginalDataService originalDataService;
    private final FollowBizService followBizService;

    @Autowired
    public OriginDataDealTask(final OriginalDataService originalDataService,
                              final FollowBizService followBizService) {
        this.originalDataService = originalDataService;
        this.followBizService = followBizService;
    }

    @Scheduled(initialDelay = 1000 * 15, fixedRate = 1000 * 15)
    public void syn2Task() {
        List<OriginalData> list = originalDataService.list(Wrappers.<OriginalData>lambdaQuery()
                .eq(OriginalData::getSolved, Boolean.FALSE)
                .last("limit 0,500")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (OriginalData data : list) {
            data.setSolved(followBizService.origin2task(data, data.getType()));
        }
        originalDataService.updateBatchById(list);
    }
}
