package cn.fw.valhalla.component;

import cn.fw.passport.sdk.enums.SubScribeEventEnum;
import cn.fw.passport.sdk.mq.SubscribeEvent;
import cn.fw.valhalla.domain.dto.WechatSubscribeSaveDTO;
import cn.fw.valhalla.service.data.WechatSubscribeService;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;


/**
 * 监听关注公众号mq
 * @author kurisu
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = "subs_event", consumerGroup = "${spring.application.name}" + "-subs_event")
public class AttentionConsumer implements RocketMQListener<SubscribeEvent> {
    private final WechatSubscribeService wechatSubscribeService;

    @Autowired
    public AttentionConsumer(final WechatSubscribeService wechatSubscribeService) {
        this.wechatSubscribeService = wechatSubscribeService;
    }

    @Override
    public void onMessage(SubscribeEvent t) {
        log.info("监听关注公众号mq: SubscribeEvent=[{}]", t);
        SubScribeEventEnum eventType = t.getEventType();
        try {
            if (eventType.equals(SubScribeEventEnum.SUBSCRIBE)) {
                final WechatSubscribeSaveDTO dto = new WechatSubscribeSaveDTO();
                dto.setMemberId(t.getUserId());
                dto.setSubscribed(Boolean.TRUE);
                dto.setSubscribeTime(new Date());
                wechatSubscribeService.saveOrUpdate(dto);
            } else {
                final WechatSubscribeSaveDTO dto = new WechatSubscribeSaveDTO();
                dto.setMemberId(t.getUserId());
                dto.setSubscribed(Boolean.FALSE);
                dto.setUnSubscribeTime(new Date());
                wechatSubscribeService.saveOrUpdate(dto);
            }
        } catch (Exception ex) {
            log.error("消费关注公众号mq失败，原因:{}", JSON.toJSONString(ex));
        }
    }
}