package cn.fw.valhalla.component;

import cn.fw.card.sdk.api.mq.CardMessageDto;
import cn.fw.card.sdk.enums.CardStatusEnum;
import cn.fw.card.sdk.enums.CardTypeEnum;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.enums.CusTagEnum;
import cn.fw.valhalla.service.data.CustomerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author Kurisu
 * 2020-08-27 4:42 下午
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = "card_notify", consumerGroup = "${spring.application.name}" + "-card_notify")
public class CardNotifyConsumer implements RocketMQListener<CardMessageDto> {
    private final CustomerService customerService;

    @Autowired
    CardNotifyConsumer(final CustomerService customerService) {
        this.customerService = customerService;
    }

    @Override
    public void onMessage(final CardMessageDto c) {
        log.info("接收到卡系统MQ消息,{}", c);
        try {
            //更新客户标签
            updateCustomer(c);
        } catch (Exception e) {
            log.info("卡系统MQ消息消费失败", e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateCustomer(CardMessageDto c) {
        Customer customer = customerService.queryById(c.getCusId());
        if (customer == null) {
            return;
        }
        List<String> tagList = StringUtils.isEmpty(customer.getTags()) ? new ArrayList<>() : Arrays.asList(customer.getTags().split(","));
        Set<String> tagSet = new HashSet<>(tagList);
        //质保卡
        if (CardTypeEnum.WARRANTY_CARD.getValue().equals(c.getCardType())) {
            tagSet.add(CusTagEnum.QUA_ASS_CARD.getValue());

            if (CardStatusEnum.INVALID.getValue().equals(c.getCardStatus())) {
                tagSet.remove(CusTagEnum.QUA_ASS_CARD.getValue());
            }
        }
        //套餐卡
        if (CardTypeEnum.MAINTAIN_CARD.getValue().equals(c.getCardType())) {
            tagSet.add(CusTagEnum.MAINTAIN_CARD.getValue());

            if (CardStatusEnum.INVALID.getValue().equals(c.getCardStatus())) {
                tagSet.remove(CusTagEnum.MAINTAIN_CARD.getValue());
            }
        }
        String newTags = String.join(",", tagSet);
        Customer newCustomer = new Customer();
        newCustomer.setId(customer.getId());
        newCustomer.setTags(newTags);
        customerService.updateById(newCustomer);
    }
}
