package cn.fw.valhalla.component;

import cn.fw.cas.sdk.param.UpdateMileageDto;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.service.data.CustomerService;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;


/**
 * 监听关注公众号mq
 *
 * @author kurisu
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = "customer_update_mileage", consumerGroup = "${spring.application.name}" + "-customer_update_mileage")
public class MileageConsumer implements RocketMQListener<UpdateMileageDto> {
    private final CustomerService customerService;

    @Autowired
    public MileageConsumer(final CustomerService customerService) {
        this.customerService = customerService;
    }

    @Override
    public void onMessage(UpdateMileageDto t) {
        log.info("监听档案里程变化mq: UpdateMileageDto=[{}]", t);
        try {
            Customer customer = new Customer();
            customer.setId(t.getCustomerId());
            customer.setCurrentMileage(t.getMileage());
            customer.setUpdateTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
            customerService.updateById(customer);
        } catch (Exception ex) {
            log.error("消费档案里程变化mq失败，原因:{}", JSON.toJSONString(ex));
        }
    }
}