package cn.fw.valhalla.component;

import cn.fw.erp.sdk.api.enums.OperateTypeEnum;
import cn.fw.erp.sdk.api.mq.RoleChangeEvent;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.dto.RoleChangeDTO;
import cn.fw.valhalla.domain.enums.LeaveReasonEnum;
import cn.fw.valhalla.domain.enums.LeaveTodoTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : RoleChangeConsumer
 * @description : 角色变动mq
 * @date: 2021-01-28 09:49
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = RoleChangeEvent.TOPIC, consumerGroup = "${spring.application.name}-" + RoleChangeEvent.TOPIC)
public class RoleChangeConsumer implements RocketMQListener<RoleChangeEvent> {
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:mq:role")
    @Getter
    private String keyPrefix;

    @Autowired
    public RoleChangeConsumer(final StringRedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    @Override
    public void onMessage(RoleChangeEvent t) {
        log.info("监听角色变动mq: RoleChangeEvent=[{}]", t);
        try {
            if (Objects.isNull(t)) {
                return;
            }
            boolean bool = OperateTypeEnum.BLOCK.getValue().equals(t.getType()) || OperateTypeEnum.REMOVE.getValue().equals(t.getType());
            if (bool && RoleCode.FWGW.equalsIgnoreCase(t.getRoleCode())) {
                String jsonString = JSONObject.toJSONString(new RoleChangeDTO(t.getRangeValue(), t.getUserId(), t.getUserName()));
                redisTemplate.boundListOps(getRedisKey(RoleCode.FWGW)).rightPush(jsonString);
            }

            if (bool && RoleCode.XBGJ.equalsIgnoreCase(t.getRoleCode())) {
                String jsonString = JSONObject.toJSONString(new RoleChangeDTO(t.getRangeValue(), t.getUserId(), t.getUserName()));
                redisTemplate.boundListOps(getRedisKey(RoleCode.XBGJ)).rightPush(jsonString);
            }
        } catch (Exception ex) {
            log.error("消费角色变动mq失败，原因:{}", JSON.toJSONString(ex));
        }
    }

    private String getRedisKey(final String roleCode) {
        return String.format("%s:change:%s", getKeyPrefix(), roleCode);
    }
}
