package cn.fw.valhalla.service.bus;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.sdk.api.ImSendMessage;
import cn.fw.hermes.sdk.api.para.MsgPara;
import cn.fw.hermes.sdk.api.para.MsgParamCondition;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.dto.DistributableDTO;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.dto.RoleChangeDTO;
import cn.fw.valhalla.domain.dto.StammkundeDto;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.DistributableVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.rpc.backlog.TodoRpcService;
import cn.fw.valhalla.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : LeaveNeedDoBizService
 * @description : 人员变动处理
 * @date: 2020-10-16 17:04
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class LeaveNeedDoBizService {
    private final LeaveNeedDoService leaveNeedDoService;
    private final CustomerService customerService;
    private final UserService userService;
    private final EhrRpcService ehrRpcService;
    private final OopService oopService;
    private final FollowBizService followBizService;
    private final FollowTaskService followTaskService;
    private final FollowRecordService followRecordService;
    private final ImSendMessage imSendMessage;
    private final TodoRpcService todoRpcService;
    private final CustomerCluePoolService customerCluePoolService;
    private final StammkundePoolService stammkundePoolService;

    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:LeaveNeedDo")
    @Getter
    private String keyPrefix;

    @Value("${follow.todo.leave2do}")
    @Getter
    private String leave2doCode;


    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#shopId + ':add:' + #userId", message = "请勿重复提交")
    public void add(final Long userId, final Long shopId) {
        List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
        LeaveNeedDo db = addable(userId, shopId);
        leaveNeedDoService.save(db);
        push2Todo(db.getId(), postUserDTOS.get(0).getUserId(), shopId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void add(LeaveNeedDo leaveNeedDo) {
        Long userId = leaveNeedDo.getUserId();
        Long shopId = leaveNeedDo.getShopId();
        List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
        boolean bool = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        ) > 0;
        if (bool) {
            return;
        }
        int count = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        );
        if (count <= 0) {
            return;
        }
        leaveNeedDoService.save(leaveNeedDo);
        Map<String, String> map = new HashMap<>();
        map.put("id", String.valueOf(leaveNeedDo.getId()));
        map.put("shopId", String.valueOf(shopId));
        map.put("userName", leaveNeedDo.getUserName());
        map.put("customerNum", String.valueOf(count));
        map.put("leaveTime", DateUtil.getFullDateString(leaveNeedDo.getEffectiveTime(), "-"));
        push2Todo(leaveNeedDo.getId(), postUserDTOS.get(0).getUserId(), shopId, map);
    }

    public AppPage<LeaveNeedDoVO> getList(LoginAuthBean currentUser, LeaveQueryVO queryVO) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(currentUser.getUserId(), RoleCode.BYKFP);
        List<Long> shopIds = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
        BV.isFalse(CollectionUtils.isEmpty(shopIds), () -> "无权限操作，请检查角色权限是否正确");
        PageData<LeaveNeedDo> pageData = leaveNeedDoService.page(new PageData<>(queryVO), Wrappers.<LeaveNeedDo>lambdaQuery()
                .in(LeaveNeedDo::getShopId, shopIds)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
                .eq(Objects.nonNull(queryVO.getType()), LeaveNeedDo::getType, queryVO.getType())
        );
        AppPage<LeaveNeedDoVO> page = AppPage.empty(queryVO);
        List<LeaveNeedDoVO> list = new ArrayList<>();
        List<LeaveNeedDo> records = Optional.ofNullable(pageData.getRecords()).orElse(new ArrayList<>());
        for (LeaveNeedDo needDo : records) {
            int i = customerService.count(Wrappers.<Customer>lambdaQuery()
                    .eq(Customer::getAdviserId, needDo.getUserId())
                    .eq(Customer::getYn, Boolean.TRUE)
            );
            list.add(LeaveNeedDoVO.with(needDo, i));
        }
        page.setData(list);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#dto.id", message = "请勿重复提交")
    public void allocation(LoginAuthBean user, LeaveAllocationDTO dto) {
        final String key = generateKey(dto.getId());
        try {
            //准备分配 常规校验
            prepareAllocation(dto);
            //变更档案
            List<Customer> customerList = doAllocation(dto, key);
            //分配跟进任务
            dealFollowTask(customerList);

            List<FollowTypeEnum> enumList = Arrays.asList(FollowTypeEnum.FM, FollowTypeEnum.RM);
            List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                    .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                    .eq(FollowTask::getFollowUser, dto.getAdviserId())
                    .in(FollowTask::getType, enumList)
            );
            if (!CollectionUtils.isEmpty(taskList)) {
                for (FollowTask followTask : taskList) {
                    dealTask(followTask);
                }
            }
            //完成分配发送消息提醒等
            finish(user, dto.getId(), key);
        } finally {
            clearKey(key);
        }
    }


    @Transactional(rollbackFor = Exception.class)
    public void xbgjChanged(RoleChangeDTO dto) {
        final Long userId = dto.getUserId();
        final Long shopId = dto.getShopId();
        List<ClueStatusEnum> statusList = Arrays.asList(ClueStatusEnum.WAITING, ClueStatusEnum.ONGOING);
        List<CustomerCluePool> list = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                .in(CustomerCluePool::getClueStatus, statusList)
                .eq(CustomerCluePool::getOriginalShopId, shopId)
                .eq(CustomerCluePool::getClueType, FollowTypeEnum.IR)
                .eq(CustomerCluePool::getRedistribution, Boolean.FALSE)
                .eq(CustomerCluePool::getOriginalUserId, userId)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<PostUserDTO> userList = userService.getUserByRole(shopId, RoleCode.XBGJ);
        BV.isNotEmpty(userList, () -> "更换跟进人员失败：没有更多续保跟进员");
        Collections.shuffle(userList);
        PostUserDTO userDTO = userList.stream().filter(u -> !u.getUserId().equals(userId)).findAny().orElse(null);
        BV.notNull(userDTO, () -> "更换跟进人员失败：没有更多续保跟进员");

        for (CustomerCluePool clue : list) {
            if (ClueStatusEnum.WAITING.equals(clue.getClueStatus())) {
                clue.setOriginalUserId(userDTO.getUserId());
                clue.setOriginalUserName(userDTO.getUserName());
                clue.setOriginalShopId(shopId);
                Optional.ofNullable(oopService.shop(shopId)).ifPresent(shop -> clue.setOriginalShopName(shop.getShortName()));
                continue;
            }
            dealTask(clue, userDTO.getUserId(), shopId);
        }

        customerCluePoolService.updateBatchById(list);

        List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .eq(FollowTask::getFollowUser, userId)
                .eq(FollowTask::getType, FollowTypeEnum.IR)
        );
        if (CollectionUtils.isEmpty(taskList)) {
            return;
        }
        for (FollowTask followTask : taskList) {
            dealTask(followTask);
        }
    }

    public List<DistributableVO> distributableList(Long shopId) {
        List<DistributableVO> list = new ArrayList<>();
        List<Customer> customerList = distributableCustomer(shopId);
        if (CollectionUtils.isEmpty(customerList)) {
            return list;
        }
        Map<Long, List<Customer>> userCustomer = customerList.stream().collect(Collectors.groupingBy(Customer::getAdviserId));
        for (Map.Entry<Long, List<Customer>> entry : userCustomer.entrySet()) {
            DistributableVO vo = new DistributableVO();
            StaffInfoDTO staffInfoDTO = ehrRpcService.queryStaffInfo(entry.getKey());
            vo.setUserId(entry.getKey());
            vo.setUserName(Optional.ofNullable(staffInfoDTO).map(StaffInfoDTO::getName).orElse(""));
            vo.setDistributableNum(entry.getValue().size());
            vo.setShopId(shopId);
            list.add(vo);
        }
        return list;
    }


    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'shop:' + #dto.shopId", message = "请勿重复提交")
    public void activeAllocation(LoginAuthBean user, DistributableDTO dto) {
        //准备分配 常规校验
        Map<Long, List<Customer>> map = verifyAndPrepare(dto);
        //变更档案
        List<Customer> list = activeDoAllocation(dto, map);
        int size = list.size();
        if (size <= 0) {
            return;
        }
        //分配非续保跟进任务
        dealFollowTask(list);
        //处理续保任务
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(dto.getAdviserId(), RoleCode.XBGJ);
        if (!CollectionUtils.isEmpty(dataRange) && dataRange.stream().anyMatch(r -> r.getRangeValue().equals(dto.getShopId()))) {
            for (Customer customer : list) {
                List<CustomerCluePool> cluePoolList = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                        .eq(CustomerCluePool::getRefererId, customer.getId())
                        .eq(CustomerCluePool::getClueStatus, ClueStatusEnum.WAITING)
                        .eq(CustomerCluePool::getGroupId, customer.getGroupId())
                        .eq(CustomerCluePool::getClueType, FollowTypeEnum.IR)
                );
                dealClue(cluePoolList, dto.getAdviserId(), dto.getShopId());
            }
        }
        //完成分配发送消息提醒等
        try {
            String text = String.format("%s通过资源分配给你%s台保有客", user.getUserName(), size);
            final MsgParamCondition msgPara = MsgParamCondition.getOfflineTxetPara(text, null, "保有客分配",
                    "保有客分配", null, dto.getAdviserId()).build();
            final Message<Boolean> msg = imSendMessage.synSendMsg(msgPara);
            log.info("给[{}]推送im消息结果:[{}]", dto.getAdviserName(), msg.getResult());
        } catch (Exception e) {
            log.error("给[{}]推送im消息失败：]", dto.getAdviserName(), e);
        }
    }

    private void prepareAllocation(LeaveAllocationDTO dto) {
        LeaveNeedDo needDo = leaveNeedDoService.queryProcessableById(dto.getId());
        BV.notNull(needDo, () -> "该条记录已处理或不存在，请刷新后重试");
        dto.setAdviserId(needDo.getUserId());

        List<LeaveAllocationDTO.Allocation> ruleList = dto.getRuleList();
        BV.notNull(ruleList, () -> "分配规则不能为空");

        for (LeaveAllocationDTO.Allocation rule : ruleList) {
            List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(rule.getUserId(), RoleCode.FWGW);
            List<Long> shopIdList = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
            BV.isNotEmpty(shopIdList, () -> String.format("[%s]无服务接待角色，请核对", rule.getUserName()));
            Long shopId = shopIdList.get(0);
            BV.isTrue(needDo.getShopId().equals(shopId), () -> String.format("[%s]所属门店与离职人员门店不符", rule.getUserName()));
            rule.setShopId(shopId);
        }
    }

    /**
     * 处理数据
     *
     * @param dto
     * @return 处理完的档案
     */
    private List<Customer> doAllocation(LeaveAllocationDTO dto, String key) {
        List<Customer> customerList = customerService.queryByAdviserId(dto.getAdviserId());
        if (CollectionUtils.isEmpty(customerList)) {
            return null;
        }
        List<LeaveAllocationDTO.Allocation> ruleList = dto.getRuleList();

        final List<Customer> list = new ArrayList<>();
        final List<StammkundePool> spl = new ArrayList<>();
        final LinkedList<UserInfo> queue = new LinkedList<>();

        for (LeaveAllocationDTO.Allocation rule : ruleList) {
            int size = customerList.size();
            List<Customer> subList = customerList.subList(0, rule.getNum() > size ? size : rule.getNum());
            if (CollectionUtils.isEmpty(subList)) {
                continue;
            }
            for (Customer customer : subList) {
                customer.setShopId(rule.getShopId());
                customer.setAdviserId(rule.getUserId());
                rejectPool(customer.getId(), rule.getShopId(), rule.getUserName(), rule.getUserId(), customer.getGroupId());
                spl.add(createPool(customer, rule.getUserName()));
                list.add(customer);
            }
            queue.add(new UserInfo(rule.getUserId(), rule.getUserName(), subList.size()));
            subList.clear();
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        setToCache(key, queue);
        return list;
    }


    /**
     * 完成分配的后续处理逻辑
     *
     * @param list
     */
    private void dealFollowTask(List<Customer> list) {
        List<FollowTypeEnum> enumList = Arrays.asList(FollowTypeEnum.FM, FollowTypeEnum.RM);
        List<ClueStatusEnum> statusList = Arrays.asList(ClueStatusEnum.WAITING, ClueStatusEnum.ONGOING);
        if (!CollectionUtils.isEmpty(list)) {
            //处理待办等信息
            for (Customer customer : list) {
                final Long customerId = customer.getId();
                final Long shopId = customer.getShopId();
                final Long adviserId = customer.getAdviserId();
                List<CustomerCluePool> cluePoolList = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                        .eq(CustomerCluePool::getRefererId, customerId)
                        .in(CustomerCluePool::getClueStatus, statusList)
                        .eq(CustomerCluePool::getGroupId, customer.getGroupId())
                        .in(CustomerCluePool::getClueType, enumList)
                );

                dealClue(cluePoolList, adviserId, shopId);
            }
        }
    }

    private void dealClue(List<CustomerCluePool> list, Long adviserId, Long shopId) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (CustomerCluePool clue : list) {
            if (ClueStatusEnum.WAITING.equals(clue.getClueStatus())) {
                clue.setOriginalUserId(adviserId);
                Optional.ofNullable(userService.user(adviserId)).ifPresent(user -> clue.setOriginalUserName(user.getUserName()));
                clue.setOriginalShopId(shopId);
                Optional.ofNullable(oopService.shop(shopId)).ifPresent(shop -> clue.setOriginalShopName(shop.getShortName()));
                continue;
            }
            dealTask(clue, adviserId, shopId);
        }

        customerCluePoolService.updateBatchById(list);
    }

    private void dealTask(CustomerCluePool clue, Long userId, Long shopId) {
        FollowTask task = followTaskService.queryOngoingTaskByClueId(clue.getId());
        if (Objects.isNull(task)) {
            return;
        }
        UserInfoDTO user = userService.user(userId);
        String userName = Objects.nonNull(user) ? user.getUserName() : "";
        task.setCloseTime(new Date());
        task.setState(TaskStateEnum.DEFEAT);
        task.setReason(TaskDefeatTypeEnum.D);
        FollowRecord taskRecord = followRecordService.getOne(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, task.getId())
                .eq(FollowRecord::getAddTodo, Boolean.FALSE)
                .isNull(FollowRecord::getFollowTime)
                .last(" limit 1")
        );
        followTaskService.updateById(task);
        if (Boolean.TRUE.equals(task.getRedistribution())) {
            clue.setCloseTime(new Date());
            clue.setClueStatus(ClueStatusEnum.FAILURE);
            return;
        } else {
            clue.setRedistribution(Boolean.TRUE);
        }
        FollowTask nTask = new FollowTask();
        nTask.setClueId(clue.getId());
        nTask.setCustomerId(clue.getRefererId());
        nTask.setType(clue.getClueType());
        nTask.setState(TaskStateEnum.ONGOING);
        nTask.setBeginTime(new Date());
        nTask.setRedistribution(Boolean.TRUE);
        nTask.setDeadline(clue.getDeadline());
        nTask.setFollowUser(userId);
        nTask.setFollowUserName(userName);
        nTask.setFollowShop(shopId);
        nTask.setGroupId(clue.getGroupId());
        followTaskService.save(nTask);
        if (Objects.nonNull(taskRecord)) {
            taskRecord.setTaskId(nTask.getId());
            taskRecord.setUserId(nTask.getFollowUser());
            taskRecord.setUserName(nTask.getFollowUserName());
            followRecordService.updateById(taskRecord);
            return;
        }
        followBizService.startTask(nTask);
    }

    private void dealTask(FollowTask task) {
        CustomerCluePool cluePool = customerCluePoolService.getById(task.getClueId());
        task.setCloseTime(new Date());
        task.setState(TaskStateEnum.DEFEAT);
        task.setReason(TaskDefeatTypeEnum.D);
        followTaskService.updateById(task);
        followRecordService.removeByTaskId(task.getId());
        if (Objects.isNull(cluePool)) {
            return;
        }
        cluePool.setCloseTime(new Date());
        cluePool.setClueStatus(ClueStatusEnum.FAILURE);
        customerCluePoolService.updateById(cluePool);
    }


    private void finish(LoginAuthBean user, Long leaveId, String key) {
        leaveNeedDoService.dealById(leaveId);
        List<UserInfo> list = getAllFromCache(key);
        for (UserInfo info : list) {
            try {
                String text = String.format("%s通过资源分配给你%s台保有客", user.getUserName(), info.getCount());
                final MsgPara msgPara = MsgPara.getOfflineTxetPara(text, null, "保有客分配",
                        "保有客分配", null, info.getUserId()).build();
                final Message<Integer> msg = imSendMessage.sendMsg(msgPara);
                log.info("给[{}]推送im消息结果:[{}]", info.getUserName(), msg.getResult());
            } catch (Exception e) {
                log.error("给[{}]推送im消息失败：]", info, e);
            }
        }
        clearKey(key);
        BackLogItemDTO backLogItemDTO = new BackLogItemDTO(user.getUserId(), getLeave2doCode(), String.valueOf(leaveId), new Date(), null);
        todoRpcService.complete(backLogItemDTO);
    }

    private void rejectPool(Long customerId, Long shopId, String userName, Long userId, Long groupId) {
        StammkundeDto stammkundeDto = new StammkundeDto();
        stammkundeDto.setCustomerId(customerId);
        stammkundeDto.setReason(DefeatReasonEnum.LC);
        stammkundeDto.setShopId(shopId);
        stammkundeDto.setNewUserId(userId);
        stammkundeDto.setNewUserName(userName);
        ShopDTO shop = oopService.shop(shopId);
        stammkundeDto.setNewShopId(shopId);
        if (Objects.nonNull(shop)) {
            stammkundeDto.setNewShopName(shop.getShortName());
        }
        stammkundeDto.setGroupId(groupId);

        stammkundePoolService.reject(stammkundeDto);
    }

    private StammkundePool createPool(Customer customer, String userName) {
        StammkundePool sp = new StammkundePool();
        sp.setCustomerId(customer.getId());
        sp.setAdviserId(customer.getAdviserId());
        sp.setAdviserName(userName);
        sp.setSources(StammkundeSourcesEnum.SYSTEMATIC);
        sp.setPoolStatus(StammkundeStatusEnum.KUNDE);
        sp.setAktiv(Boolean.TRUE);
        sp.setActivationTime(new Date());
        sp.setShopId(customer.getShopId());
        sp.setGroupId(customer.getGroupId());
        return sp;
    }

    private String generateKey(final Long leaveId) {
        Assert.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:allocation:%s", getKeyPrefix(), leaveId);
    }

    public List<UserInfo> getAllFromCache(final String key) {
        try {
            Boolean hasKey = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(hasKey)) {
                return new ArrayList<>();
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            List<String> stringList = ops.range(0, -1);
            if (CollectionUtils.isEmpty(stringList)) {
                return new ArrayList<>();
            }
            List<UserInfo> dtos = stringList.stream().map(str -> JSONObject.parseObject(str, UserInfo.class)).collect(Collectors.toList());
            redisTemplate.delete(key);
            return dtos;
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
            return new ArrayList<>();
        }
    }

    private void setToCache(final String key, final UserInfo userInfo) {
        try {
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String jsonString = JSONObject.toJSONString(userInfo);
            ops.rightPush(jsonString);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void setToCache(final String key, final LinkedList<UserInfo> userInfos) {
        try {
            if (CollectionUtils.isEmpty(userInfos)) {
                return;
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String[] users = userInfos.stream().map(JSONObject::toJSONString).toArray(String[]::new);
            ops.rightPushAll(users);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void clearKey(final String key) {
        try {
            redisTemplate.delete(key);
        } catch (Exception e) {
            log.error("清空缓存信息失败 key[{}]", key, e);
        }
    }

    private LeaveNeedDo addable(final Long userId, final Long shopId) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(userId, RoleCode.FWGW);
        boolean ok = CollectionUtils.isEmpty(dataRange) || !Objects.equals(dataRange.get(0).getRangeValue(), shopId);
        BV.isTrue(ok, () -> "请先移除对对应服务站服务顾问角色后操作");
        int count = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        );
        BV.isTrue(count == 0, () -> "已存在待分配记录，请勿重复添加");
        UserInfoDTO user = userService.user(userId);
        BV.notNull(user, () -> "用户不存在");
        boolean bool = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        ) > 0;
        BV.isTrue(bool, () -> "该顾问没有可用档案无需分配保有客");
        LeaveNeedDo db = new LeaveNeedDo();
        db.setDone(Boolean.FALSE);
        db.setEffectiveTime(DateUtil.getMonthEndDay(new Date()));
        db.setReason(LeaveReasonEnum.OTHER);
        db.setType(LeaveTodoTypeEnum.CUSTOMER);
        db.setShopId(shopId);
        db.setUserId(userId);
        db.setUserName(user.getUserName());
        db.setCreateTime(new Date());
        db.setUpdateTime(new Date());
        return db;
    }

    private void push2Todo(Long id, Long userId, Long shopId) {
        Map<String, String> map = new HashMap<>();
        map.put("shopId", String.valueOf(shopId));
        map.put("id", String.valueOf(id));
        push2Todo(id, userId, shopId, map);
    }

    private void push2Todo(Long id, Long userId, Long shopId, Map<String, String> map) {
        BackLogItemDTO dto = new BackLogItemDTO(userId, getLeave2doCode(), String.valueOf(id), new Date(), shopId);
        dto.setDynamicMap(map);
        todoRpcService.push(dto);
    }

    private List<Customer> distributableCustomer(Long shopId) {
        Set<Long> customerIdSet = new HashSet<>();
        List<CustomerCluePool> cluePoolList = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getOriginalShopId, shopId)
                .eq(CustomerCluePool::getClueStatus, ClueStatusEnum.ONGOING)
                .ne(CustomerCluePool::getClueType, FollowTypeEnum.AC)
        );
        if (!CollectionUtils.isEmpty(cluePoolList)) {
            customerIdSet.addAll(cluePoolList.stream().map(CustomerCluePool::getRefererId).collect(Collectors.toSet()));
        }
        List<FollowTask> followTaskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getFollowShop, shopId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .ne(FollowTask::getType, FollowTypeEnum.AC)
        );
        if (!CollectionUtils.isEmpty(followTaskList)) {
            customerIdSet.addAll(followTaskList.stream().map(FollowTask::getCustomerId).collect(Collectors.toSet()));
        }

        return customerService.list(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getYn, Boolean.TRUE)
                .eq(Customer::getShopId, shopId)
                .notIn(!CollectionUtils.isEmpty(customerIdSet), Customer::getId, customerIdSet)
                .isNull(Customer::getTags)
                .isNull(Customer::getCusLevel)
        );
    }

    private Map<Long, List<Customer>> verifyAndPrepare(DistributableDTO dto) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(dto.getAdviserId(), RoleCode.FWGW);
        List<Long> shopIdList = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
        BV.isNotEmpty(shopIdList, () -> String.format("[%s]无服务接待角色，请核对", dto.getAdviserName()));

        List<DistributableDTO.BeAssignedDTO> beAssignedList = dto.getBeAssignedList();
        BV.isNotEmpty(beAssignedList, () -> "分配规则不能为空");

        List<Customer> customerList = distributableCustomer(dto.getShopId());
        BV.isNotEmpty(customerList, () -> "无可分配的档案");

        final Map<Long, List<Customer>> map = customerList.stream().collect(Collectors.groupingBy(Customer::getAdviserId));
        beAssignedList.forEach(c -> {
            int size = Optional.ofNullable(map.get(c.getUserId())).orElse(new ArrayList<>()).size();
            Integer distributableNum = c.getDistributableNum();
            Integer num = c.getNum();
            if (size < distributableNum) {
                c.setDistributableNum(size);
            }
            if (size < num) {
                c.setDistributableNum(size);
                c.setNum(size);
            }
        });

        List<DistributableDTO.BeAssignedDTO> dtoList = beAssignedList.stream().filter(r -> r.getDistributableNum() <= 0 || r.getDistributableNum() < r.getNum()).collect(Collectors.toList());
        BV.isEmpty(dtoList, () -> "分配规则有误，请刷新页面重试");
        return map;
    }


    /**
     * 处理数据
     *
     * @param dto
     * @param map
     * @return 处理完的档案
     */
    private List<Customer> activeDoAllocation(DistributableDTO dto, Map<Long, List<Customer>> map) {
        List<Customer> customerList = new ArrayList<>();
        List<DistributableDTO.BeAssignedDTO> assignedList = dto.getBeAssignedList();
        for (DistributableDTO.BeAssignedDTO assignedDTO : assignedList) {
            customerList.addAll(map.get(assignedDTO.getUserId()).subList(0, assignedDTO.getNum()));
        }
        if (CollectionUtils.isEmpty(customerList)) {
            return null;
        }
        Long shopId = dto.getShopId();
        Long adviserId = dto.getAdviserId();
        String adviserName = dto.getAdviserName();

        final List<Customer> list = new ArrayList<>();
        final List<StammkundePool> spl = new ArrayList<>();

        for (Customer customer : customerList) {
            customer.setShopId(shopId);
            customer.setAdviserId(adviserId);
            rejectPool(customer.getId(), shopId, adviserName, adviserId, customer.getGroupId());
            spl.add(createPool(customer, adviserName));
            list.add(customer);
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        return list;
    }

    static class UserInfo {
        private Long userId;
        private String userName;
        private int count;

        public UserInfo() {
        }

        public UserInfo(Long userId, String userName) {
            this.userId = userId;
            this.userName = userName;
        }

        public UserInfo(Long userId, String userName, int count) {
            this.userId = userId;
            this.userName = userName;
            this.count = count;
        }

        public Long getUserId() {
            return userId;
        }

        public void setUserId(Long userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public int getCount() {
            return count;
        }

        public void setCount(int count) {
            this.count = count;
        }

        @Override
        public String toString() {
            return "UserInfo{" +
                    "userId=" + userId +
                    ", userName='" + userName + '\'' +
                    ", count=" + count +
                    '}';
        }
    }
}
