package cn.fw.valhalla.service.bus;

import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerReachLog;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.sdk.param.ReachLogReq;
import cn.fw.valhalla.service.data.CustomerReachLogService;
import cn.fw.valhalla.service.data.CustomerService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author : kurisu
 * @className : ReachLogBizService
 * @description : 进站记录BIZ
 * @date: 2020-11-11 17:48
 */
@Service
@RequiredArgsConstructor
public class ReachLogBizService {
    private final CustomerService customerService;
    private final CustomerReachLogService customerReachLogService;
    private final UserService userService;


    public void saveReachLog(ReachLogReq reachLogReq) {
        Long customerId = reachLogReq.getCustomerId();
        Customer customer = customerService.getById(customerId);
        if (Objects.isNull(customer)) {
            return;
        }
        CustomerReachLog reachLog = new CustomerReachLog();
        BeanUtils.copyProperties(reachLogReq, reachLog);
        reachLog.setFrameNo(customer.getFrameNo());
        reachLog.setAdvisorId(customer.getAdviserId());
        UserInfoDTO user = userService.user(customer.getAdviserId());
        if (Objects.nonNull(user)) {
            reachLog.setAdvisorId(user.getId());
            reachLog.setAdvisorName(user.getUserName());
        }
        customerReachLogService.save(reachLog);
        int count = Objects.nonNull(customer.getArrivalCount()) ? customer.getArrivalCount() : 0;
        customer.setArrivalTime(reachLogReq.getArrivalTime());
        customer.setArrivalCount(count + 1);
        customer.setCurrentMileage(reachLogReq.getArrivalMileage());
        customerService.updateById(customer);
    }
}
