package cn.fw.valhalla.service.bus.setting.strategy.impl;

import cn.fw.valhalla.domain.dto.SettingDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.SettingTypeEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.service.bus.setting.strategy.AbstractSettingStrategy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : ACSetting
 * @description : 事故车设置
 * @date: 2020-08-14 10:28
 */
@Component
@Slf4j
public class ACSetting extends AbstractSettingStrategy {
    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.AC;
    }

    @Override
    public List<SettingVO> getList(Long groupId) {
        return this.getListByCat(getFollowType(), groupId);
    }

    @Override
    public boolean save(List<SettingDTO> list, Long groupId) {
        return this.saveOrUpdate(getFollowType(), groupId, list);
    }

    @Override
    public SettingVO getBySettingType(SettingTypeEnum settingTypeEnum, Long groupId) {
        return getBySettingType(getFollowType(), groupId, settingTypeEnum);
    }

    @Override
    protected boolean saveOrUpdate(FollowTypeEnum type, Long groupId, List<SettingDTO> dtoList) {
        //事故车只有这两种配置
        final Integer[] types = {SettingTypeEnum.LIMITATION.getValue(), SettingTypeEnum.EFFECTIVE_TIME.getValue(), SettingTypeEnum.FAIL_TIME.getValue()};
        final Set<Integer> set = Arrays.stream(types).collect(Collectors.toSet());
        List<SettingDTO> list = dtoList.stream().filter(r -> set.contains(r.getType())).collect(Collectors.toList());
        return super.saveOrUpdate(type, groupId, list);
    }
}
