package cn.fw.valhalla.service.data;

import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.dto.FollowPoolDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.query.FollowPoolQueryVO;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * @author : kurisu
 * @className : FollowTaskService
 * @description : 跟进任务
 * @date: 2020-08-12 11:04
 */
public interface FollowTaskService extends IService<FollowTask> {
    /**
     * 根据档案id查询
     *
     * @param customerId
     * @param list
     * @return
     */
    List<FollowTask> getWaitListByCustomerId(Long customerId, List<FollowTypeEnum> list);

    /**
     * 跟进池查询
     *
     * @param queryVO
     * @return
     */
    List<FollowPoolDTO> followList(FollowPoolQueryVO queryVO);

    /**
     * 跟进池查询总数
     *
     * @param queryVO
     * @return
     */
    long followListCount(FollowPoolQueryVO queryVO);

    /**
     * 查询客户所有的跟进信息
     *
     * @param customerId
     * @param list
     * @return
     */
    List<FollowTask> getListByCustomerId(Long customerId, List<FollowTypeEnum> list);

    /**
     * 根据线索id查询正在进行中的任务
     *
     * @param clueId
     * @return
     */
    FollowTask queryOngoingTaskByClueId(Long clueId);

    /**
     * 查询是否有正在进行中的跟进任务
     *
     * @param customerId
     * @param followType
     * @return
     */
    boolean hasOngoingTask(Long customerId, Integer followType);
}
