package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.CustomerCluePoolMapper;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.dto.CustomerCluePoolDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.query.CustomerCluePoolQueryVO;
import cn.fw.valhalla.service.data.CustomerCluePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @className : CustomerCluePoolServiceImpl
 * @description : 线索池
 * @date: 2020-11-12 09:36
 */
@Service
public class CustomerCluePoolServiceImpl extends ServiceImpl<CustomerCluePoolMapper, CustomerCluePool> implements CustomerCluePoolService {
    @Override
    @Nullable
    public CustomerCluePool queryByRefererId(Long refererId, Long groupId, FollowTypeEnum type) {
        return getOne(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getRefererId, refererId)
                .eq(CustomerCluePool::getGroupId, groupId)
                .eq(CustomerCluePool::getClueType, type)
                .orderByDesc(CustomerCluePool::getId)
                .last(" limit 1")
        );
    }

    @Override
    @Nullable
    public CustomerCluePool queryByVin(String vin, Long groupId, FollowTypeEnum type) {
        return getOne(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getFrameNo, vin)
                .eq(CustomerCluePool::getGroupId, groupId)
                .eq(CustomerCluePool::getClueType, type)
                .orderByDesc(CustomerCluePool::getId)
                .last(" limit 1")
        );
    }

    @Override
    @Nullable
    public CustomerCluePool queryByPlateNo(String plateNo, Long groupId, FollowTypeEnum type) {
        return getOne(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getPlateNo, plateNo)
                .eq(CustomerCluePool::getGroupId, groupId)
                .eq(CustomerCluePool::getClueType, type)
                .orderByDesc(CustomerCluePool::getId)
                .last(" limit 1")
        );
    }

    @Override
    public List<CustomerCluePoolDTO> clueList(CustomerCluePoolQueryVO queryVO) {
        Integer current = queryVO.getCurrent();
        Integer pageSize = queryVO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().clueList(startIndex, pageSize, queryVO)).orElse(new ArrayList<>());
    }

    @Override
    public Long clueListCount(CustomerCluePoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().clueListCount(queryVO)).orElse(0L);
    }

    @Override
    public Long completeNum(CustomerCluePoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().completeNum(queryVO)).orElse(0L);
    }

    @Override
    public Long defeatNum(CustomerCluePoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().defeatNum(queryVO)).orElse(0L);
    }

    @Override
    public Long onGoingNum(CustomerCluePoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().onGoingNum(queryVO)).orElse(0L);
    }
}
