package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.FollowTaskMapper;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.dto.FollowPoolDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.domain.query.FollowPoolQueryVO;
import cn.fw.valhalla.service.data.FollowTaskService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @className : FollowTaskServiceImpl
 * @description : 跟进任务
 * @date: 2020-08-12 11:05
 */
@Service
@Slf4j
public class FollowTaskServiceImpl extends ServiceImpl<FollowTaskMapper, FollowTask> implements FollowTaskService {
    @Override
    public List<FollowTask> getWaitListByCustomerId(Long customerId, List<FollowTypeEnum> list) {
        return Optional.ofNullable(list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getCustomerId, customerId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .in(!CollectionUtils.isEmpty(list), FollowTask::getType, list)
        )).orElse(new ArrayList<>());
    }

    @Override
    public List<FollowPoolDTO> followList(FollowPoolQueryVO queryVO) {
        Integer current = queryVO.getCurrent();
        Integer pageSize = queryVO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().followList(startIndex, pageSize, queryVO)).orElse(new ArrayList<>());
    }

    @Override
    public long followListCount(FollowPoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().followListCount(queryVO)).orElse(0L);
    }

    @Override
    public List<FollowTask> getListByCustomerId(Long customerId, List<FollowTypeEnum> list) {
        return Optional.ofNullable(list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getCustomerId, customerId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .in(!CollectionUtils.isEmpty(list), FollowTask::getType, list)
        )).orElse(new ArrayList<>());
    }

    @Override
    public FollowTask queryOngoingTaskByClueId(Long clueId) {
        return getOne(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getClueId, clueId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .last(" limit 1")
        );
    }

    @Override
    public boolean hasOngoingTask(Long customerId, Integer followType) {
        return count(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getCustomerId, customerId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .eq(FollowTask::getType, followType)
        ) > 0;
    }
}
