package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.PublicCluePoolMapper;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.dto.PublicCluePoolDTO;
import cn.fw.valhalla.domain.enums.PublicClueStateEnum;
import cn.fw.valhalla.domain.query.PublicCluePoolQueryVO;
import cn.fw.valhalla.service.data.PublicCluePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @className : PublicCluePoolServiceImpl
 * @description : 公共池客户线索池
 * @date: 2021-03-29 15:48
 */
@Service
public class PublicCluePoolServiceImpl extends ServiceImpl<PublicCluePoolMapper, PublicCluePool> implements PublicCluePoolService {
    @Override
    public List<PublicCluePoolDTO> publicCluePoolList(PublicCluePoolQueryVO queryVO) {
        Integer current = queryVO.getCurrent();
        Integer pageSize = queryVO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().publicCluePoolList(startIndex, pageSize, queryVO)).orElse(new ArrayList<>());
    }

    @Override
    public long publicCluePoolListCount(PublicCluePoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().publicCluePoolListCount(queryVO)).orElse(0L);
    }

    @Override
    public void removeClue(Long customerId, Long adviserId) {
        update(Wrappers.<PublicCluePool>lambdaUpdate()
                .set(PublicCluePool::getState, PublicClueStateEnum.DEFEAT)
                .set(PublicCluePool::getCloseTime, new Date())
                .eq(PublicCluePool::getCustomerId, customerId)
                .eq(PublicCluePool::getAdviserId, adviserId)
                .isNull(PublicCluePool::getCloseTime)
        );
    }

    @Override
    public void closeClue(Long customerId, Long adviserId) {
        update(Wrappers.<PublicCluePool>lambdaUpdate()
                .set(PublicCluePool::getState, PublicClueStateEnum.COMPLETE)
                .set(PublicCluePool::getCloseTime, new Date())
                .eq(PublicCluePool::getCustomerId, customerId)
                .eq(PublicCluePool::getAdviserId, adviserId)
                .isNull(PublicCluePool::getCloseTime)
        );
    }

    @Override
    public PublicCluePool queryByAdviserAndCust(Long customerId, Long adviserId) {
        return this.getOne(Wrappers.<PublicCluePool>lambdaQuery()
                .eq(PublicCluePool::getCustomerId, customerId)
                .eq(PublicCluePool::getAdviserId, adviserId)
                .last(" limit 1 ")
        );
    }
}
