package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.PublicPoolMapper;
import cn.fw.valhalla.domain.db.pool.PublicPool;
import cn.fw.valhalla.domain.dto.PublicPoolDTO;
import cn.fw.valhalla.domain.query.PublicPoolQueryVO;
import cn.fw.valhalla.service.data.PublicPoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @className : PublicPoolServiceImpl
 * @description : 公共池
 * @date: 2020-08-12 11:22
 */
@Slf4j
@Service
public class PublicPoolServiceImpl extends ServiceImpl<PublicPoolMapper, PublicPool> implements PublicPoolService {
    @Override
    public Optional<PublicPool> queryByPlate(String plateNo, Long groupId) {
        PublicPool pool = getOne(Wrappers.<PublicPool>lambdaQuery()
                .eq(PublicPool::getGroupId, groupId)
                .eq(PublicPool::getPlateNo, plateNo)
                .last("limit 1")
        );
        return Optional.ofNullable(pool);
    }

    @Override
    public void removeByCustomerId(Long customerId, Long groupId) {
        remove(Wrappers.<PublicPool>lambdaQuery()
                .eq(PublicPool::getCustomerId, customerId)
                .eq(PublicPool::getGroupId, groupId)
        );
    }

    @Override
    public List<PublicPoolDTO> publicList(PublicPoolQueryVO queryVO) {
        Integer current = queryVO.getCurrent();
        Integer pageSize = queryVO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().publicList(startIndex, pageSize, queryVO)).orElse(new ArrayList<>());
    }

    @Override
    public Long publicListCount(PublicPoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().publicListCount(queryVO)).orElse(0L);
    }
}
